/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 2009 Toporov Konstantin. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 3 only ("GPL")  (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.gnu.org/copyleft/gpl.html  See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * You should include file containing license in each project.
 * If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * If you wish your version of this file to be governed by
 * the GPL Version 3, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [GPL Version 3] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under the GPL Version 3 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 3 code and therefore, elected the GPL
 * Version 3 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 *
 * Contributor(s):
 *
 * Portions Copyrighted 2010 Toporov Konstantin.
 */

package org.diacalc.mobile;

/**
 *
 * @author Toporov Konstantin <www.diacalc.org>
 */

import com.sun.lwuit.*;
import com.sun.lwuit.events.ActionEvent;
import com.sun.lwuit.events.ActionListener;
import com.sun.lwuit.events.DataChangedListener;
import com.sun.lwuit.events.SelectionListener;
import com.sun.lwuit.layouts.BorderLayout;
import com.sun.lwuit.layouts.BoxLayout;
import com.sun.lwuit.list.DefaultListModel;
import com.sun.lwuit.list.ListCellRenderer;
import com.sun.lwuit.plaf.Border;
import com.sun.lwuit.plaf.UIManager;
import java.util.Hashtable;
import java.util.Vector;
import javax.microedition.midlet.MIDlet;
import org.diacalc.mobile.components.*;
import org.diacalc.mobile.maths.DoingPOST;
import org.diacalc.mobile.maths.Ordering;
import org.diacalc.mobile.maths.Str;
import org.diacalc.mobile.products.ProductGroup;
import org.diacalc.mobile.products.ProductInBase;
import org.diacalc.mobile.products.ProductW;


public class ProdsForm extends FormBase implements ActionListener{
    public String getName() {
        return "Продукты";
    }
    public String getIconName(){
        return "prods";
    }
    private FishEyeTickerRenderer prRenderer = new FishEyeTickerRenderer();
    private static final int N_GR_COMMAND = 10;
    private static final int E_GR_COMMAND = 11;
    private static final int D_GR_COMMAND = 12;
    private static final int N_PR_COMMAND = 13;
    private static final int E_PR_COMMAND = 14;
    private static final int D_PR_COMMAND = 15;
    private static final int YES_COMMAND = 16;
    private static final int NO_COMMAND = 17;
    private static final int SEARCH_COMMAND = 18;
    private static final int IMPORT_BASE_COMMAND = 19;
    private static final int EXPORT_BASE_COMMAND = 20;

    private static final Command newGrCommand = new Command("Новая группа", N_GR_COMMAND);
    private static final Command editGrCommand = new Command("Изм. группу", E_GR_COMMAND);
    private static final Command delGrCommand = new Command("Удал. группу", D_GR_COMMAND);

    private static final Command newPrCommand = new Command("Новый прод.", N_PR_COMMAND);
    private static final Command editPrCommand = new Command("Изм. прод.", E_PR_COMMAND);
    private static final Command delPrCommand = new Command("Удал. прод.", D_PR_COMMAND);
    private static final Command searchCommand = new Command("Поиск", SEARCH_COMMAND);
    
    private static final Command importBaseCommand = new Command("Импорт", IMPORT_BASE_COMMAND);
    private static final Command exportBaseCommand = new Command("Экспорт", EXPORT_BASE_COMMAND);

    private static final Command yesCommand = new Command("Да",YES_COMMAND);
    private static final Command noCommand = new Command("Нет",NO_COMMAND);

    //components
    private TextField filter;
    private HorizontalList groups;
    private DefaultListModel prModel;
    private FilterProxyListModel proxyModel;
    private List prodsL;
    private Container bottomPane;

    private DCJmobile master = null;

    private boolean searchMode;
    private boolean flagGrDelete;
    

    public void cleanup(){
        filter = null;
        searchMode = false;
        flagGrDelete = false;
        bottomPane = null;
        groups = null;
        prModel = null;
        proxyModel = null;
        searchMode = false;
        flagGrDelete = false;
    }
    
    private ContextMenu contextMenuProduct = new ContextMenu();
    
    protected void execute(final MIDlet m, boolean refresh) {
        if (contextMenuProduct.getCommandsSize()==0){
            contextMenuProduct.addCommand(editPrCommand,this);
            contextMenuProduct.addCommand(delPrCommand,this);
        }
        if (refresh){
            cleanup();
            master = (DCJmobile)m;
            
            form.getContentPane().setLayout(new BorderLayout());
            
            groups = new HorizontalList(new DefaultListModel( master.getGroups() ));
            groups.setFocusable(false);

            prModel = new DefaultListModel( master.getProds() );

            proxyModel = new FilterProxyListModel(prModel);
            prodsL = new List(proxyModel){
                public void longPointerPress(int x, int y){
                    super.longPointerPress(x, y);
                    //Тут показываем контекстное меню.
                    if (contextMenuProduct.getCommandsSize()==0 || 
                            prodsL.getModel().getSize()==0 ||
                            prodsL.getSelectedIndex()<0 || 
                            prodsL.getSelectedIndex()>(prodsL.getModel().getSize()-1)) return;
                    //Container 
                    contextMenuProduct.showContextMenu( 
                            ((ProductInBase)prodsL.getSelectedItem()).getName()
                            );
                }
            };
            
            groups.setSelectedIndex(0);

            proxyModel.filter("",((ProductGroup)groups.getSelectedItem()).getId());

            groups.addSelectionListener(new SelectionListener(){
                public void selectionChanged(int o,int n){
                    if (o!=n || flagGrDelete){
                        clearFilter();
                        proxyModel.filter("",
                               ((ProductGroup)groups.getSelectedItem()).getId());
                        if (prodsL.getModel().getSize()>0) prodsL.setSelectedIndex(0, true);
                        if (flagGrDelete) flagGrDelete = false;
                    }
                }

            });

            prodsL.addActionListener(new ActionListener(){
                public void actionPerformed(ActionEvent ev){
                    ProductInBase prod = (ProductInBase)prodsL.getSelectedItem();
                    //вот тут если включен режим поиска нужно оформить выход из него
                    if (searchMode) searching();
                    else if (prod!=null){
                        //changemenu = true;
                        prod.setSelected(!prod.isSelected());
                        if (prod.isSelected()){
                            master.addProductToMenu(prod);
                        }
                        else{
                            master.removeProductFromMenu(prod);
                        }
                    }
                }
            });
            prodsL.setFireOnClick(false);

            
            form.getContentPane().addComponent(BorderLayout.NORTH,groups);
            form.getContentPane().addComponent(BorderLayout.CENTER,prodsL);

            prodsL.setFixedSelection(List.FIXED_NONE);//двигаемся по кругу "карусель"

            prodsL.setPaintFocusBehindList(true);
            prodsL.setRenderer(prRenderer);
            
            prodsL.addSelectionListener(new SelectionListener(){
                public void selectionChanged(int o,int n){
                    if (o!=n){
                        prRenderer.restartTicker();
                    }
                }
             });

            form.getContentPane().setScrollable(false);

            form.addMenuCommand(searchCommand,this);
            form.addMenuCommand(delGrCommand,this);
            form.addMenuCommand(editGrCommand,this);
            form.addMenuCommand(newGrCommand,this);
            
            form.addMenuCommand(importBaseCommand,this);
            form.addMenuCommand(exportBaseCommand, this);

            form.addMenuCommand(newPrCommand,this);

            
            bottomPane = new Container(new BorderLayout());
            Button btn2menu = new Button(" * ");
            btn2menu.addActionListener(new ActionListener(){
                public void actionPerformed(ActionEvent ev){
                    master.showForm(DCJmobile.MENU, false);
                }
            });
            btn2menu.setFocusable(false);

            createFilter();

            bottomPane.addComponent(BorderLayout.CENTER, filter);
            bottomPane.addComponent(BorderLayout.EAST, btn2menu);
            form.getContentPane().addComponent(BorderLayout.SOUTH,bottomPane);
            
        }
    }
    
    private void createFilter(){
        filter = new TextField();
        filter.addDataChangeListener(new DataChangedListener() {
            public void dataChanged(int type, int index) {
                proxyModel.filter(filter.getText(),
                    ((ProductGroup)groups.getSelectedItem()).getId());
            }
        });

        filter.addActionListener(new ActionListener(){
            public void actionPerformed(ActionEvent ev){
                if (filter.getText().length()==0){
                    proxyModel.filter("",
                        ((ProductGroup)groups.getSelectedItem()).getId());
                    clearFilter();
                }
            }
        });
    
        VirtualKeyboard.bindVirtualKeyboard(filter, DCJmobile.createRusTxtKeyboard(false));
    }
    
    private void clearFilter(){//Сейчас тут не скрытие фильтра, а его очистка, НЕ УБИРАТЬ!
        if (filter==null && bottomPane==null) return;//Первый запуск
        if (!form.contains(filter) && !form.contains(bottomPane)) return;

        filter.setText("");

        prodsL.requestFocus();
        proxyModel.filter("",
                    ((ProductGroup)groups.getSelectedItem()).getId());
    }
    
    public void actionPerformed(ActionEvent ev){
        switch (ev.getCommand().getId()) {
            case N_GR_COMMAND: addGroup(); break;
            case E_GR_COMMAND: editGroup(); break;
            case D_GR_COMMAND: deleteGroup(); break;
            case N_PR_COMMAND: addProduct(); break;
            case D_PR_COMMAND: deleteProduct(); break;
            case E_PR_COMMAND: editProduct(); break;
            case SEARCH_COMMAND: searching(); break;
            case IMPORT_BASE_COMMAND: importBase(); break;
            case EXPORT_BASE_COMMAND: exportBase(); break;
        }
    }
    
    private Hashtable inetRequestResults;
    
    private void exportBase(){
        if (master.getGroups().isEmpty() || master.getProds().isEmpty()) return;
        Hashtable ver_check = null;
        
        final Vector exGr = new Vector();
        final Vector exPr = new Vector();
        //Добавляем продукты и группы
        for(int p=0;p<master.getProds().size();p++){
            if ( ((ProductInBase)master.getProds().elementAt(p)).isMobile() ){
                exPr.addElement( master.getProds().elementAt(p) );
                boolean found = false;
                for(int g=0;g<exGr.size();g++){
                    if ( ((ProductInBase)master.getProds().elementAt(p)).getOwner()==
                            ((ProductGroup)exGr.elementAt(g)).getId() ){
                        found = true;
                        break;
                    }
                }
                if (!found){
                    //Добавляем группу
                    for(int g=0;g<master.getGroups().size();g++){
                        if ( ((ProductInBase)master.getProds().elementAt(p)).getOwner()==
                            ((ProductGroup)master.getGroups().elementAt(g)).getId() ){
                            exGr.addElement( master.getGroups().elementAt(g) );
                            break;
                        }
                    }
                }
            }
        }
        //есть что отправлять
        if (exPr.isEmpty()) return;
        
        //Больше ничего не мешает, начинаем
        final InfoDialog infoDlg = new InfoDialog();
        Display.getInstance().callSeriallyAndWait(new Runnable(){
            public void run(){
                infoDlg.showModeless();
            }
        });
        
        //Отправляем
        Display.getInstance().invokeAndBlock(new Runnable() {
            public void run() {
                // perform IO operation..
                inetRequestResults = new DoingPOST(master.getUser()).sendProducts(exGr, exPr);
            }
        });
        
        String response = (String)inetRequestResults.get(DoingPOST.ERROR);
        
        if (response.equals(DoingPOST.NO_ERROR)){
            //тут проверяем версию программы
            ver_check = new DoingPOST(master.getUser()).checkVersion();
        }
        
        Display.getInstance().callSeriallyAndWait(new Runnable(){
            public void run(){
                infoDlg.dispose();
            }
        });
        
        if (response.equals(DoingPOST.NO_ERROR)){
            //Все успешно, показываем поздравление
            TextArea txt = new TextArea("Продукты успешно выгружены на сервер");
            txt.setEditable(false);
            Dialog.show("Сообщение",
                            txt,
                            new Command[] {new Command("Ok")},
                            Dialog.TYPE_INFO,
                            null);
            VersionInfo.showVersionInfo(ver_check);
        }
        else{
            //произошла ошибка
            TextArea txt = new TextArea("При выгрузке продуктов произошли ошибки\n"+
                    Str.replace(response,"<br>","\n"));
            txt.setEditable(false);
            Dialog.show("Ошибка",
                            txt,
                            new Command[] {new Command("Ok")},
                            Dialog.TYPE_ERROR,
                            null);
        }
    }
    private void importBase(){
        Hashtable ver_check = null;
        
        final InfoDialog infoDlg = new InfoDialog();
        Display.getInstance().callSeriallyAndWait(new Runnable(){
            public void run(){
                infoDlg.showModeless();
            }
        });
        //infoDlg.showModeless();
        //А сама загрузка должна быть в отдельном потоке
        Display.getInstance().invokeAndBlock(new Runnable() {
            public void run() {
                // perform IO operation..
                inetRequestResults = new DoingPOST(master.getUser()).receiveProducts();
            }
        });
        
        String response = (String)inetRequestResults.get(DoingPOST.ERROR);
        
        if (response.equals(DoingPOST.NO_ERROR)){
            //тут проверяем версию программы
            ver_check = new DoingPOST(master.getUser()).checkVersion();
        }
        
        
        Display.getInstance().callSeriallyAndWait(new Runnable(){
            public void run(){
                infoDlg.dispose();
            }
        });
        
        if (response.equals(DoingPOST.NO_ERROR)){
            TextArea txt = new TextArea("Продукты загружены успешно\nТекущая база продуктов будет заменена\n"
                    + "Продолжить?" );
            txt.setEditable(false);
            
            if (Dialog.show("Сообщение",
                txt,
                new Command[]{yesCommand,noCommand} ).getId()==YES_COMMAND){
                
                clearFilter();
                
                master.getProds().removeAllElements();
                master.getGroups().removeAllElements();
                
                groups.setSelectedIndex(-1);
                
                Vector prods = (Vector)inetRequestResults.get(DoingPOST.PRODS_PRODS);
                Ordering.setSortField(Ordering.NAME_FIELD);
                Ordering.sort(prods);
                for(int i=0;i<prods.size();i++){
                    prodsL.addItem(prods.elementAt(i));
                }
                
                Vector grps = (Vector)inetRequestResults.get(DoingPOST.PRODS_GROUP);
                for(int i=0;i<grps.size();i++){
                    groups.addItem(grps.elementAt(i));
                }
                
                if (grps.size()>0) groups.setSelectedIndex(0);
                
                master.setNeedSaveBase();
            }
            VersionInfo.showVersionInfo(ver_check);
        }
        else{
            //произошла ошибка
            TextArea txt = new TextArea("При загрузке продуктов произошли ошибки\n"+
                    Str.replace(response,"<br>","\n"));
            txt.setEditable(false);
            Dialog.show("Ошибка",
                            txt,
                            new Command[] {new Command("Ok")},
                            Dialog.TYPE_ERROR,
                            null);
        }
    }
    private void searching(){
        if (searchMode){
            //Отменяем режим поиска
            //Нужно получить продукт, найтие его группу, и выделить его потом в списке
            ProductInBase pr = (ProductInBase)prodsL.getSelectedItem();

            groups.setSearch(false);//Сначала отключили в гор. списке
            clearFilter();
            searchMode = false;

            int i=0;
            for(;i<master.getGroups().size();i++){
                if ( ((ProductGroup)master.getGroups().elementAt(i)).getId()==
                        pr.getOwner()){
                    break;
                }
            }
            if (i<master.getGroups().size()){
                groups.setSelectedIndex(i);
                for(int j=0;j<prodsL.size();j++){
                    if ( ((ProductInBase)prodsL.getModel().getItemAt(j)).getId()==pr.getId() ){
                        prodsL.setSelectedIndex(j, true);
                        break;
                    }
                }
            }
        }else{
            //Включаем поиск
            groups.setSearch(true);
            searchMode = true;
            
            String st = filter.getText();
            
            filter.requestFocus();
            proxyModel.filter(st,-1);
            if (prodsL.size()>0) prodsL.setSelectedIndex(0, true);
        }
        
    }
    private void deleteProduct(){
        if (searchMode) return;
        if (prodsL.size()==0) return;
        ProductInBase p = (ProductInBase)prodsL.getSelectedItem();
        TextArea txt = new TextArea("Удалить продукт:\n" +
                p.getName()+"?");
        txt.setEditable(false);
        if (Dialog.show("Удал. продукт",
                txt,
                new Command[]{yesCommand,noCommand} ).getId()==YES_COMMAND){
            master.getProds().removeElement(p);
            String f = "";
            if (filter!=null){
                f = filter.getText();
            }
            proxyModel.filter(f,
                         p.getOwner() );
            master.setNeedSaveBase();
        }
    }
    private void editProduct(){
        if (searchMode) return;
        if (prodsL.size()==0) return;
        if (prodsL.getSelectedIndex()<0) return;
        ProductInBase old = (ProductInBase)prodsL.getSelectedItem();
        ProductEditor c = new ProductEditor( old ,master);
        
        int res = Dialog.show("Изм. продукт",
                c,
                new Command[]{yesCommand,noCommand} ).getId();

        if (res==YES_COMMAND){
            ProductInBase pr = c.getProduct();
            pr.setWeight(100f);
            if (pr.getName().length()>0){//Начинаем изменять
                old.setName(pr.getName());
                old.setProt(pr.getProt());
                old.setFat(pr.getFat());
                old.setCarb(pr.getCarb());
                old.setGi(pr.getGi());
                old.setMobile(true);
                old.setOwner(pr.getOwner());
                
                Ordering.setSortField(Ordering.NAME_FIELD);
                Ordering.setSortType(Ordering.SORT_BY_ASC);
                Ordering.sort(master.getProds());

                int pos = -1;
                for(int i=0;i<master.getGroups().size();i++){
                    if ( ((ProductGroup)master.getGroups().elementAt(i)).getId() ==
                            pr.getOwner() ){
                        pos = i;
                        break;
                    }
                }
                groups.setSelectedIndex(pos);
                clearFilter();

                proxyModel.filter("",
                         pr.getOwner() );
                pos = -1;
                for(int i=0;i<proxyModel.getSize();i++){
                    if ( ((ProductInBase)proxyModel.getItemAt(i)).getId()==old.getId() ){
                        pos = i;
                        break;
                    }
                }
                if (pos>=0){
                    prodsL.setSelectedIndex(pos, true);
                }
                master.setNeedSaveBase();
            }
        }
    }
    private void addProduct(){
        if (searchMode) return;
        ProductW prN = new ProductW();
        prN.setWeight(100f);
        ProductEditor c = new ProductEditor(new ProductInBase(prN,
                ((ProductGroup)groups.getModel().getItemAt(groups.getSelectedIndex())).getId()
                ),
                master);
        
        int res = Dialog.show("Добавить продукт",
                c,
                new Command[]{yesCommand,noCommand} ).getId();

        if (res==YES_COMMAND){
            ProductInBase pr = c.getProduct();
            pr.setWeight(100f);
            if (pr.getName().length()>0){
                prModel.addItem( pr );
                Ordering.setSortField(Ordering.NAME_FIELD);
                Ordering.sort(master.getProds());

                int pos = -1;
                for(int i=0;i<master.getGroups().size();i++){
                    if ( ((ProductGroup)master.getGroups().elementAt(i)).getId() ==
                            pr.getOwner() ){
                        pos = i;
                        break;
                    }
                }
                groups.setSelectedIndex(pos);
                clearFilter();

                proxyModel.filter("",
                         pr.getOwner() );
                pos = -1;
                for(int i=0;i<proxyModel.getSize();i++){
                    if ( ((ProductInBase)proxyModel.getItemAt(i)).getId()==pr.getId() ){
                        pos = i;
                        break;
                    }
                }
                if (pos>=0){
                    prodsL.setSelectedIndex(pos, true);
                }
                master.setNeedSaveBase();
            }
        }
    }
    private void editGroup(){
        if (searchMode) return;
        Container c = new Container(new BoxLayout(BoxLayout.Y_AXIS));
        Label lbl = new Label("Название группы");
        TextField fld = new TextField(
                ((ProductGroup)groups.getSelectedItem()).getName() );
        
        VirtualKeyboard.bindVirtualKeyboard(fld, DCJmobile.createRusTxtKeyboard(true));
        
        c.addComponent(lbl);
        c.addComponent(fld);

        Command res = Dialog.show("Изм. группу", c,new Command[]{yesCommand,noCommand} );
        if (res.getId()==YES_COMMAND && fld.getText().length()>0){
            ((ProductGroup)groups.getSelectedItem()).setName(fld.getText());
            int ind = groups.getSelectedIndex();
            groups.setSelectedIndex(ind);
            master.setNeedSaveBase();
        }
    }
    private void addGroup(){
        if (searchMode) return;
        Container c = new Container(new BoxLayout(BoxLayout.Y_AXIS));
        Label lbl = new Label("Название группы");
        TextField fld = new TextField( "" );
        
        VirtualKeyboard.bindVirtualKeyboard(fld, DCJmobile.createRusTxtKeyboard(true));
        
        c.addComponent(lbl);
        c.addComponent(fld);

        Command res = Dialog.show("Нов. группа", c,new Command[]{yesCommand,noCommand} );
        if (res.getId()==YES_COMMAND && fld.getText().length()>0){
            groups.addItem(new ProductGroup(fld.getText(),-1));
            groups.setSelectedIndex(groups.getModel().getSize()-1);
            proxyModel.filter("",
                         ((ProductGroup)groups.getSelectedItem()).getId());
            master.setNeedSaveBase();
        }
    }
    private void deleteGroup(){
        if (searchMode) return;
        if (groups.size()<2) return;
        clearFilter();

        TextArea txt = new TextArea("Будет удалена группа "
                + ((ProductGroup)groups.getSelectedItem()).getName() +
                " и все продукты в нее входящие");
        txt.setEditable(false);
        if (Dialog.show("Удл. группы",
                txt,
                new Command[]{yesCommand,noCommand} ).getId() == YES_COMMAND){
            //Удаляем
            int selO = groups.getSelectedIndex();
            
            int sel = 0;
            //тут надо сменить прокси модель заранее
            //если удаляем последний, то выбираем предыдущий, если первый, то выбираем следующий
            if (selO>0) sel = selO - 1;
            else sel = 1;

            int id = ((ProductGroup)groups.getSelectedItem()).getId();
            //удаляем все продукты из этой группы
            for(int i=master.getProds().size();i>0;i--){
                if ( ((ProductInBase)master.getProds().elementAt(i-1)).getOwner()==id ){
                    master.getProds().removeElementAt(i-1);
                }
            }

            groups.getModel().removeItem(groups.getSelectedIndex());

            //вот тут надо шагнуть назад или вперед
            flagGrDelete = true;
            if (selO>0) groups.setSelectedIndex(sel);//шаг назад
            else groups.setSelectedIndex(0);//т.к. удаляли последний, то возврат на шаг
            
            master.setNeedSaveBase();
        }

    }
    

    protected String getHelp() {
        return "";
        /*return "В данном окне можно выбрать продукты для меню, для этого " +
                "необходимо отметить галкой нужный продукт.\n" +
                "Для перемещения по группам используйте джойстик вправо и влево, " +
                "либо на touch screen'ах нажимайте стрелки справа и слева от названия группы.\n" +
                "Для быстрой фильтрации продуктов начинайте набирать на клавиатуре " +
                "буквы, которые есть в составе названия искомого продукта.\n" +
                "Для быстрого перемещения в меню нажмите *";*/
    }

    /**
     * Demonstrates implementation of a renderer derived from a label
     * and the use of icons in the renderer
     */
    private static class FishEyeTickerRenderer extends CheckBox implements ListCellRenderer{
        private Label label = new Label();

      private AnimatedCheckBoxRenderer selectedRenderer = new AnimatedCheckBoxRenderer();
      private List parentList;
      private Container cont = new Container(new BoxLayout(BoxLayout.Y_AXIS));
      private AnimatedRenderer descr = new AnimatedRenderer(false);

      public FishEyeTickerRenderer() {
          super();
          setFocusable(false);
          label.getStyle().setBgTransparency(100);
          getStyle().setBgTransparency(0);

          selectedRenderer.setTickerEnabled(true);
          selectedRenderer.setFocus(true);
          selectedRenderer.getSelectedStyle().setBgTransparency(0);
          selectedRenderer.setFocusable(false);
          descr.setUIID("LabelProducts");
          descr.setTickerEnabled(true);
          descr.getSelectedStyle().setBgTransparency(0);
          descr.setFocus(true);
          descr.setFocusable(true);

          cont.addComponent(selectedRenderer);
          cont.addComponent(descr);

          cont.getStyle().setBorder(Border.createLineBorder(1));
          cont.getStyle().setBgTransparency(255);
          cont.getStyle().setBgColor(0xFFFFFF);
      }
      public void restartTicker(){
          descr.resetPosition();
          selectedRenderer.resetPosition();
      }
      public boolean animate() {
          if(parentList != null && parentList.getComponentForm() != null) {
              if(selectedRenderer.isTickerRunning()) {
                  if(selectedRenderer.animate()) {
                      parentList.repaint();
                  }
              }
              if(descr.isTickerRunning()) {
                  if(descr.animate()) {
                      parentList.repaint();
                  }
              }
          }
          return super.animate();
      }

      public Component getListCellRendererComponent(List list, Object value, int index, boolean isSelected) {
          ProductInBase pr = (ProductInBase)value;
          if(isSelected) {
              selectedRenderer.setSelected(pr.isSelected());
              selectedRenderer.getListCellRendererComponent(list, pr.getName(), index, isSelected);
              descr.getListCellRendererComponent(list,
                        "Б:"+Str.round(pr.getProt(),1)+
                        " Ж:"+Str.round(pr.getFat(),1) +
                        " У:"+Str.round(pr.getCarb(),1) +
                        " ГИ:"+pr.getGi(),
                      index, isSelected);
                      //.setText();
              // sometimes the list asks for a dummy selected value for size calculations and this might
              // break the tickering state
              if(index == list.getSelectedIndex()) {
                  if(selectedRenderer.shouldTickerStart()) {
                      if(!selectedRenderer.isTickerRunning()) {
                          parentList = list;
                          list.getComponentForm().registerAnimated(this);
                          selectedRenderer.startTicker(UIManager.getInstance().getLookAndFeel().getTickerSpeed(), true);
                      }
                  } else {
                      if(selectedRenderer.isTickerRunning()) {
                          selectedRenderer.stopTicker();
                      }
                  }
                  if (descr.shouldTickerStart()){
                      if (!descr.isTickerRunning()){
                          parentList = list;
                          list.getComponentForm().registerAnimated(this);
                          descr.startTicker(UIManager.getInstance().getLookAndFeel().getTickerSpeed(), true);
                      }
                  }
                  else{
                      if(descr.isTickerRunning()) {
                          descr.stopTicker();
                      }
                  }
              }
              return cont;
          } else {
              setText(pr.getName());
              setSelected(pr.isSelected());
              return this;
          }
      }
      public Component getListFocusComponent(List list) {
        return label;
      }
      /**
        * Overriden to do nothing and remove a performance issue where renderer changes
        * perform needless repaint calls
        */
      public void repaint() {
      }
  }
}
