/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 2009 Toporov Konstantin. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 3 only ("GPL")  (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.gnu.org/copyleft/gpl.html  See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * You should include file containing license in each project.
 * If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * If you wish your version of this file to be governed by
 * the GPL Version 3, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [GPL Version 3] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under the GPL Version 3 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 3 code and therefore, elected the GPL
 * Version 3 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 *
 * Contributor(s):
 *
 * Portions Copyrighted 2010 Toporov Konstantin.
 */

package org.diacalc.mobile.components;

/**
 *
 * @author Toporov Konstantin <www.diacalc.org>
 */

import com.sun.lwuit.*;
import com.sun.lwuit.events.ActionEvent;
import com.sun.lwuit.events.ActionListener;
import com.sun.lwuit.layouts.BoxLayout;
import java.util.Date;
import org.diacalc.mobile.DCJmobile;
import org.diacalc.mobile.maths.DiaryUnit;
import org.diacalc.mobile.maths.Str;
import org.diacalc.mobile.maths.Sugar;
import org.diacalc.mobile.maths.User;

public class EventEditor extends Container{
//1000*60*60*24 - 1 день
//14610 между 01,01,1970 и 01.01.2010
//7669 между 01,01,2010 и 31,12,2030

   
    
    private MaskTextField time;
    private MaskTextField date;
    private TextArea comment;
    private TextField sh1;
    private TextField sh2;
    private DiaryUnit du;
    private User us;
    
    public EventEditor(DiaryUnit du,User us){
        super(new BoxLayout(BoxLayout.Y_AXIS));
        this.du = du;
        this.us = us;
        java.util.Calendar c = java.util.Calendar.getInstance();
        c.setTime(new Date(this.du.getTime()));

        time = new MaskTextField("##:##");
        time.setValidator(new TimeValidator());
        time.setValue(new Integer(c.get(java.util.Calendar.HOUR_OF_DAY)*60*60 + 
                    c.get(java.util.Calendar.MINUTE)*60));

        date = new MaskTextField("##.##.##");
        date.setValidator(new DateValidator());
        date.setValue(new Date(this.du.getTime()));
        
        comment = new TextArea(this.du.getComment(),5,20);
        comment.setHint("Комментарий");
        
        addComponent(time);
        addComponent(date);

        if (this.du.getType()==DiaryUnit.MENU || this.du.getType()==DiaryUnit.SUGAR){
            sh1 = new TextField();
            sh1.setConstraint(TextField.DECIMAL);
            sh1.setInputMode("1+2");
            sh1.setInputModeOrder(new String[]{"1+2"});
            sh1.setText(""+Str.round( 
                    new Sugar(this.du.getSh1()).getSugar(us.isMmol(), us.isPlasma()),1));
            addComponent(Str.createPair("СК1",sh1));
        }
        
        
        if (this.du.getType()==DiaryUnit.MENU){//меню СК2
            sh2 = new TextField();
            sh2.setConstraint(TextField.DECIMAL);
            sh2.setInputMode("1+2");
            sh2.setInputModeOrder(new String[]{"1+2"});
            sh2.setText(""+Str.round( 
                    new Sugar(this.du.getSh2()).getSugar(us.isMmol(), us.isPlasma()),1));
            addComponent(Str.createPair("СК2",sh2));
            sh1.setEditable(false);
            sh2.setEditable(false);
        }

        addComponent(comment);

        VirtualKeyboard.bindVirtualKeyboard(comment, DCJmobile.createRusTxtKeyboard(true));
        
        NumbersKeyboard vk = new NumbersKeyboard();
        VirtualKeyboard.bindVirtualKeyboard(time, vk);
        VirtualKeyboard.bindVirtualKeyboard(date, vk);
        if (this.du.getType()==DiaryUnit.SUGAR){
            VirtualKeyboard.bindVirtualKeyboard(sh1, vk);
        }
        vk.addCommandListener(new ActionListener(){
            public void actionPerformed(ActionEvent ev){
                if (ev.getCommand().getCommandName().equals("Ok")){
                    Display.getInstance().callSerially(new Runnable() {
                        public void run() {
                            time.setValue( time.getValue() );
                            date.setValue(date.getValue());
                        }
                    });
                }
            }
        });
        setScrollableY(true);
    }
    
    public DiaryUnit getDiaryUnit(){
        du.setComment(comment.getText());
        du.setTime(getDate());
        if (du.getType()==DiaryUnit.MENU || du.getType()==DiaryUnit.SUGAR){
            Sugar s = new Sugar();
            s.setSugar(Str.parseString(sh1, 1), us.isMmol(), us.isPlasma());
            du.setSh1( s.getValue() );
            if (du.getType()==DiaryUnit.MENU){
                s.setSugar(Str.parseString(sh2, 1), us.isMmol(), us.isPlasma());
                du.setSh2( s.getValue() );
            }
        }
        return du;
    }

    private long getDate(){
        java.util.Calendar c = java.util.Calendar.getInstance();
        Date dt = (Date)date.getValue();
        c.setTime( dt );
        
        int tm = ((Integer)time.getValue()).intValue();
        int h = tm/(60*60);
        int m = tm % (60*60) / 60;
              
        c.set(java.util.Calendar.HOUR, h);
        c.set(java.util.Calendar.MINUTE, m);
        return c.getTime().getTime();
    }

    
}
